""" 
This is one of 2 files required to run the neural chatbot in spinnaker.  The other is ChatBotInterfaceSpinn8.py
This file (SimpleChatbotAgentSpinn8.py) contains the chatbot (agent).  ChatBotInterfaceSpinn8.py contains the interface (world).  

To run the chatbot you just need to run this file (i.e. SimpleChatbotAgentSpinn8.py) as it will call the relevant functions in ChatBotInterfaceSpinn8.py.

The agent can currently generate 3 responses:
The yellow banana is in room 1 2.
The red apple is in room 1 1.
The diamond is oval.

The language, i.e. the sentences the agent can use to answer the queries, are defined in chatbotLanguage.txt
The answer/response generated for each query (queries shown in ChatBotInterfaceNest.py) is programmed/defined in
LanguageGenerator.py.

A query entered at the interface is sent
from the environment (interface) to the spinnaker board (agent/chatbot) via
a spike which activates a query cell.  The agent (spinnaker board) then processes the query and sends back an answer (sentence)
to the environment (interface) via another set of spikes.  These spikes are fired by neurons representing words, where each neuron represents a different word
(the words represented are shown in OutputWords.txt). Therefore, the interface builds the response (sentence) using the words in OutputWords.txt and
the spikes, and spike times, from the word neurons.

Communication is achieved via live injection (from world to agent) and
receivers (from agent to world) of spikes.  


"""

import sys
import time
import os
import pyNN.spiNNaker as sim
import ChatBotInterfaceSpinn8 as Interface

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from LanguageGenerator import GeneratorClass 

NumQueries=3  #number of queries

def init():
    sim.setup(timestep=neal.DELAY,
                    min_delay=neal.DELAY,
                    max_delay=neal.DELAY, debug=0)

def createQueryCells(generator):
    #create a pop of size NumQueries.  Each neuron corresponds to one query, activating a specific query cell
    #will activate a specific response (sentence)
    global NumQueries

    SPinnInputs=generator.CreateSPINNInputs(NumQueries)


    return SPinnInputs
    
def runChatBot(length):
    #this runs the spinnaker simulation
    #during this queries (spikes) are sent to the spinnaker board via live injection
    #spinnaker board (agent) will send a response back via spikes.
    
    sim.run(length)

#---main 
simName = "spinnaker"
spinnVersion = 8
LanguageFile="chatbotLanguage.txt"  #this is the file containing the chatbot's language (sentences).
OutputWordsFile="OutputWords.txt"  #where to write all of the different words comprising the language (sentences)
                                   #in the LanguageFile. 

neal = NealCoverFunctions(simName,sim,spinnVersion)
init()
fsa = FSAHelperFunctions(simName,sim,neal,spinnVersion)
generator = GeneratorClass(simName,sim,neal,spinnVersion,fsa,LanguageFile,OutputWordsFile)

SIM_LENGTH = 4000.0

 
generatorWordCells=generator.getWordCells()  #get the word population.  The spikes from these comprise the response

#this creates the query cells that, when activated, will turn on the corresponding answer (i.e. sentence and associated phrases)
QueryCells=createQueryCells(generator)


##this connects the query cells to the corresponding answers (i.e. sentence and associated phrases)
generator.connectQueryCellsToAnswers(QueryCells)


neal.nealApplyProjections()

Interface.RunInterface(generatorWordCells,generator) #set up both the spinnaker communication and
                                                     #interface

runChatBot(SIM_LENGTH)

generator.saveGenResults('results') ##save the neural generator spiking results

sim.end()

time.sleep(5.0)# wait for the system to shut down
print "Chatbot agent (sim) ended"

